@@ -43,7 +43,7 @@ Session::Session(S_Connection conn, sr_datastore_t datastore)
     int ret;
     _conn = nullptr;
     _sess = nullptr;
-    S_Deleter deleter(new Deleter(_sess));
+    auto deleter = std::make_shared<Deleter>(_sess);
 
     /* start session */
     ret = sr_session_start(conn->_conn, datastore, &_sess);
@@ -97,7 +97,7 @@ void Session::session_notif_buffer()
 
 S_Errors Session::get_error()
 {
-    S_Errors errors(new Errors());
+    auto errors = std::make_shared<Errors>();
 
     sr_get_error(_sess, &errors->_info);
     if (errors->_info == nullptr) {
@@ -149,31 +149,31 @@ libyang::S_Context Session::get_context()
 
 S_Val Session::get_item(const char *path, uint32_t timeout_ms)
 {
-    S_Val value(new Val());
+    auto value = std::make_shared<Val>();
 
     int ret = sr_get_item(_sess, path, timeout_ms, &value->_val);
     if (SR_ERR_OK == ret) {
+        if (!value->_val) {
+            return nullptr;
+        }
         value->_deleter = std::make_shared<Deleter>(value->_val);
         return value;
     }
-    if (SR_ERR_NOT_FOUND == ret) {
-        return nullptr;
-    }
     throw_exception(ret);
 }
 
 S_Vals Session::get_items(const char *xpath, uint32_t timeout_ms, const sr_get_oper_options_t opts)
 {
-    S_Vals values(new Vals());
+    auto values = std::make_shared<Vals>();
 
     int ret = sr_get_items(_sess, xpath, timeout_ms, opts, &values->_vals, &values->_cnt);
     if (SR_ERR_OK == ret) {
+        if (!values->_vals) {
+            return nullptr;
+        }
         values->_deleter = std::make_shared<Deleter>(values->_vals, values->_cnt);
         return values;
     }
-    if (SR_ERR_NOT_FOUND == ret) {
-        return nullptr;
-    }
     throw_exception(ret);
 }
 
@@ -183,11 +183,11 @@ libyang::S_Data_Node Session::get_subtree(const char *path, uint32_t timeout_ms)
 
     int ret = sr_get_subtree(_sess, path, timeout_ms, &subtree);
     if (SR_ERR_OK == ret) {
+        if (!subtree) {
+            return nullptr;
+        }
         return std::make_shared<libyang::Data_Node>(subtree, std::make_shared<libyang::Deleter>(subtree));
     }
-    if (SR_ERR_NOT_FOUND == ret) {
-        return nullptr;
-    }
     throw_exception(ret);
 }
 
@@ -197,11 +197,11 @@ libyang::S_Data_Node Session::get_data(const char *xpath, uint32_t max_depth, ui
 
     int ret = sr_get_data(_sess, xpath, max_depth, timeout_ms, opts, &data);
     if (SR_ERR_OK == ret) {
+        if (!data) {
+            return nullptr;
+        }
         return std::make_shared<libyang::Data_Node>(data, std::make_shared<libyang::Deleter>(data));
     }
-    if (SR_ERR_NOT_FOUND == ret) {
-        return nullptr;
-    }
     throw_exception(ret);
 }
 
@@ -315,7 +315,7 @@ void Session::unlock(const char *module_name)
 
 S_Iter_Change Session::get_changes_iter(const char *xpath)
 {
-    S_Iter_Change iter(new Iter_Change());
+    auto iter = std::make_shared<Iter_Change>();
 
     int ret = sr_get_changes_iter(_sess, xpath, &iter->_iter);
     if (SR_ERR_OK == ret) {
@@ -329,7 +329,7 @@ S_Iter_Change Session::get_changes_iter(const char *xpath)
 
 S_Iter_Change Session::dup_changes_iter(const char *xpath)
 {
-    S_Iter_Change iter(new Iter_Change());
+    auto iter = std::make_shared<Iter_Change>();
 
     int ret = sr_dup_changes_iter(_sess, xpath, &iter->_iter);
     if (SR_ERR_OK == ret) {
@@ -343,7 +343,7 @@ S_Iter_Change Session::dup_changes_iter(const char *xpath)
 
 S_Change Session::get_change_next(S_Iter_Change iter)
 {
-    S_Change change(new Change());
+    auto change = std::make_shared<Change>();
 
     int ret = sr_get_change_next(_sess, iter->_iter, &change->_oper, &change->_old, &change->_new);
     if (SR_ERR_OK == ret) {
@@ -357,7 +357,7 @@ S_Change Session::get_change_next(S_Iter_Change iter)
 
 S_Tree_Change Session::get_change_tree_next(S_Iter_Change iter)
 {
-    S_Tree_Change change(new Tree_Change());
+    auto change = std::make_shared<Tree_Change>();
 
     int ret = sr_get_change_tree_next(_sess, iter->_iter, &change->_oper, &change->_node, &change->_prev_value, \
             &change->_prev_list, &change->_prev_dflt);
@@ -374,7 +374,7 @@ Session::~Session() {}
 
 S_Vals Session::rpc_send(const char *path, S_Vals input, uint32_t timeout_ms)
 {
-    S_Vals output(new Vals());
+    auto output = std::make_shared<Vals>();
 
     int ret = sr_rpc_send(_sess, path, input->_vals, input->_cnt, timeout_ms, &output->_vals, &output->_cnt);
     if (ret != SR_ERR_OK) {
@@ -418,86 +418,37 @@ void Session::event_notif_send(libyang::S_Data_Node notif)
     }
 }
 
-Callback::Callback() {}
-Callback::~Callback() {}
-
-static int module_change_cb(sr_session_ctx_t *session, const char *module_name, const char *xpath, sr_event_t event, \
-        uint32_t request_id, void *private_data)
-{
-    S_Session sess(new Session(session));
-    Callback *wrap = (Callback *)private_data;
-    return wrap->module_change(sess, module_name, xpath, event, request_id, wrap->private_data["module_change"]);
-}
-static int rpc_cb(sr_session_ctx_t *session, const char *op_path, const sr_val_t *input, const size_t input_cnt, \
-        sr_event_t event, uint32_t request_id, sr_val_t **output, size_t *output_cnt, void *private_data)
-{
-    S_Session sess(new Session(session));
-    S_Vals in_vals(new Vals(input, input_cnt, nullptr));
-    S_Vals_Holder out_vals(new Vals_Holder(output, output_cnt));
-    Callback *wrap = (Callback *)private_data;
-    return wrap->rpc(sess, op_path, in_vals, event, request_id, out_vals, wrap->private_data["rpc"]);
-}
-static int rpc_tree_cb(sr_session_ctx_t *session, const char *op_path, const struct lyd_node *input, sr_event_t event, \
-        uint32_t request_id, struct lyd_node *output, void *private_data)
-{
-    S_Session sess(new Session(session));
-    libyang::S_Data_Node in_tree(new libyang::Data_Node(const_cast<struct lyd_node *>(input)));
-    libyang::S_Data_Node out_tree(new libyang::Data_Node(output));
-    Callback *wrap = (Callback *)private_data;
-    return wrap->rpc_tree(sess, op_path, in_tree, event, request_id, out_tree, wrap->private_data["rpc_tree"]);
-}
-static void event_notif_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif_type, const char *path, \
-        const sr_val_t *values, const size_t values_cnt, time_t timestamp, void *private_data)
-{
-    S_Session sess(new Session(session));
-    S_Vals vals(new Vals(values, values_cnt, nullptr));
-    Callback *wrap = (Callback *)private_data;
-    return wrap->event_notif(sess, notif_type, path, vals, timestamp, wrap->private_data["event_notif"]);
-}
-static void event_notif_tree_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif_type, \
-        const struct lyd_node *notif, time_t timestamp, void *private_data)
-{
-    S_Session sess(new Session(session));
-    libyang::S_Data_Node node(new libyang::Data_Node(const_cast<struct lyd_node *>(notif)));
-    Callback *wrap = (Callback *)private_data;
-    return wrap->event_notif_tree(sess, notif_type, node, timestamp, wrap->private_data["event_notif_tree"]);
-}
-static int oper_get_items_cb(sr_session_ctx_t *session, const char *module_name, const char *path, \
-        const char *request_xpath, uint32_t request_id, struct lyd_node **parent, void *private_data)
-{
-    int ret;
-    libyang::S_Data_Node tree;
-    S_Session sess(new Session(session));
-    Callback *wrap = (Callback *)private_data;
-    if (*parent) {
-        tree = std::make_shared<libyang::Data_Node>(*parent);
-        ret = wrap->oper_get_items(sess, module_name, path, request_xpath, request_id, tree, wrap->private_data["oper_get_items"]);
-    } else {
-        tree = std::make_shared<libyang::Data_Node>(nullptr);
-        ret = wrap->oper_get_items(sess, module_name, path, request_xpath, request_id, tree, wrap->private_data["oper_get_items"]);
-        if (tree) {
-            *parent = lyd_dup(tree->swig_node(), LYD_DUP_OPT_RECURSIVE);
-        }
-    }
-    return ret;
-}
-
 Subscribe::Subscribe(S_Session sess)
+    : sess(sess)
+    , sess_deleter(sess->_deleter)
 {
-    _sub = nullptr;
-    _sess = sess;
-    sess_deleter = sess->_deleter;
+}
+
+Subscribe::Subscribe(S_Session sess, const FdRegistration& reg, const FdUnregistration& unreg)
+    : sess(sess)
+    , sess_deleter(sess->_deleter)
+    , reg(reg)
+    , unreg(unreg)
+{
+    if (!reg) {
+        throw std::logic_error("FD registration callback is invalid");
+    }
+    if (!unreg) {
+        throw std::logic_error("FD unregistration callback is invalid");
+    }
 }
 
 Subscribe::~Subscribe()
 {
-    if (_sub && _sess) {
-        int ret = sr_unsubscribe(_sub);
+    if (ctx) {
+        if (unreg) {
+            unreg(get_event_pipe());
+        }
+        int ret = sr_unsubscribe(ctx);
         if (ret != SR_ERR_OK) {
             //this exception can't be catched
             //throw_exception(ret);
         }
-        _sub = nullptr;
     }
 
     for (unsigned int i = 0; i < wrap_cb_l.size(); i++) {
@@ -505,92 +456,240 @@ Subscribe::~Subscribe()
     }
 }
 
-void Subscribe::module_change_subscribe(const char *module_name, S_Callback callback, const char *xpath, \
-        void *private_data, uint32_t priority, sr_subscr_options_t opts)
+void Subscribe::check_custom_loop_options(sr_subscr_options_t opts)
 {
-    callback->private_data["module_change"] = private_data;
-    cb_list.push_back(callback);
+    if (reg && !(opts & SR_SUBSCR_NO_THREAD)) {
+        throw std::logic_error("Custom loop subscription not called with SR_SUBSCR_NO_THREAD.");
+    }
 
-    opts |= SR_SUBSCR_CTX_REUSE;
-    int ret = sr_module_change_subscribe(_sess->_sess, module_name, xpath, module_change_cb, callback->get(), priority, \
-            opts, &_sub);
-    if (ret != SR_ERR_OK) {
-        throw_exception(ret);
+    if (!reg && (opts & SR_SUBSCR_NO_THREAD)) {
+        throw std::logic_error("Non-custom loop subscription mustn't be called with SR_SUBSCR_NO_THREAD.");
     }
 }
 
-void Subscribe::rpc_subscribe(const char *xpath, S_Callback callback, void *private_data, uint32_t priority, \
-        sr_subscr_options_t opts)
+void Subscribe::call_reg()
 {
-    callback->private_data["rpc"] = private_data;
-    cb_list.push_back(callback);
-
-    opts |= SR_SUBSCR_CTX_REUSE;
-    int ret = sr_rpc_subscribe(_sess->_sess, xpath, rpc_cb, callback->get(), priority, opts, &_sub);
-    if (ret != SR_ERR_OK) {
-        throw_exception(ret);
+    if (reg && !reg_called) {
+        int pipe;
+        sr_get_event_pipe(ctx, &pipe);
+        reg(pipe, [this] {sr_process_events(ctx, sess->_sess, nullptr);});
+        reg_called = true;
     }
 }
 
-void Subscribe::rpc_subscribe_tree(const char *xpath, S_Callback callback, void *private_data, uint32_t priority, \
-        sr_subscr_options_t opts)
+void Subscribe::module_change_subscribe(const char *module_name, ModuleChangeCb cb, const char *xpath, uint32_t priority, sr_subscr_options_t opts)
 {
-    callback->private_data["rpc_tree"] = private_data;
-    cb_list.push_back(callback);
+    check_custom_loop_options(opts);
+    module_change_cbs.emplace_back(cb);
 
     opts |= SR_SUBSCR_CTX_REUSE;
-    int ret = sr_rpc_subscribe_tree(_sess->_sess, xpath, rpc_tree_cb, callback->get(), priority, opts, &_sub);
+    int ret = sr_module_change_subscribe(
+            sess->_sess,
+            module_name,
+            xpath,
+            [] (sr_session_ctx_t *session,
+                const char *module_name,
+                const char *xpath,
+                sr_event_e event,
+                uint32_t request_id,
+                void *private_data)
+            {
+                auto sess = std::make_shared<Session>(session);
+                auto cb = reinterpret_cast<ModuleChangeCb*>(private_data);
+                return (*cb)(sess, module_name, xpath, event, request_id);
+            },
+            (void*)&(module_change_cbs.back()),
+            priority,
+            opts,
+            &ctx);
     if (ret != SR_ERR_OK) {
         throw_exception(ret);
     }
+    call_reg();
 }
 
-void Subscribe::event_notif_subscribe(const char *module_name, S_Callback callback, const char *xpath, time_t start_time, \
-        time_t stop_time, void *private_data, sr_subscr_options_t opts)
+void Subscribe::rpc_subscribe(const char *xpath, RpcCb cb, uint32_t priority, sr_subscr_options_t opts)
 {
-    callback->private_data["event_notif"] = private_data;
-    cb_list.push_back(callback);
+    check_custom_loop_options(opts);
+    rpc_cbs.push_back(cb);
 
     opts |= SR_SUBSCR_CTX_REUSE;
-    int ret = sr_event_notif_subscribe(_sess->_sess, module_name, xpath, start_time, stop_time, event_notif_cb, \
-            callback->get(), opts, &_sub);
+    int ret = sr_rpc_subscribe(
+            sess->_sess,
+            xpath,
+            [] (sr_session_ctx_t *session,
+                const char *op_path,
+                const sr_val_t *input,
+                const size_t input_cnt,
+                sr_event_t event,
+                uint32_t request_id,
+                sr_val_t **output,
+                size_t *output_cnt,
+                void *private_data)
+            {
+                auto sess = std::make_shared<Session>(session);
+                auto in_vals = std::make_shared<Vals>(input, input_cnt, nullptr);
+                auto out_vals = std::make_shared<Vals_Holder>(output, output_cnt);
+                auto cb = reinterpret_cast<RpcCb*>(private_data);
+                return (*cb)(sess, op_path, in_vals, event, request_id, out_vals);
+            },
+            (void*)&(rpc_cbs.back()),
+            priority,
+            opts,
+            &ctx);
     if (ret != SR_ERR_OK) {
         throw_exception(ret);
     }
+    call_reg();
 }
 
-void Subscribe::event_notif_subscribe_tree(const char *module_name, S_Callback callback, const char *xpath, time_t start_time, \
-        time_t stop_time, void *private_data, sr_subscr_options_t opts)
+void Subscribe::rpc_subscribe_tree(const char *xpath, RpcTreeCb cb, uint32_t priority, sr_subscr_options_t opts)
 {
-    callback->private_data["event_notif_tree"] = private_data;
-    cb_list.push_back(callback);
+    check_custom_loop_options(opts);
+    rpc_tree_cbs.push_back(cb);
 
     opts |= SR_SUBSCR_CTX_REUSE;
-    int ret = sr_event_notif_subscribe_tree(_sess->_sess, module_name, xpath, start_time, stop_time, event_notif_tree_cb, \
-            callback->get(), opts, &_sub);
+    int ret = sr_rpc_subscribe_tree(
+            sess->_sess,
+            xpath,
+            [] (sr_session_ctx_t *session,
+                const char *op_path,
+                const struct lyd_node *input,
+                sr_event_t event,
+                uint32_t request_id,
+                struct lyd_node *output,
+                void *private_data)
+            {
+                auto sess = std::make_shared<Session>(session);
+                auto in_tree = std::make_shared<libyang::Data_Node>(const_cast<struct lyd_node *>(input));
+                auto out_tree = std::make_shared<libyang::Data_Node>(output);
+                auto cb = reinterpret_cast<RpcTreeCb*>(private_data);
+                return (*cb)(sess, op_path, in_tree, event, request_id, out_tree);
+            },
+            (void*)&(rpc_tree_cbs.back()),
+            priority,
+            opts,
+            &ctx);
     if (ret != SR_ERR_OK) {
         throw_exception(ret);
     }
+    call_reg();
 }
 
-void Subscribe::oper_get_items_subscribe(const char *module_name, const char *path, S_Callback callback, \
-        void *private_data, sr_subscr_options_t opts)
+void Subscribe::event_notif_subscribe(const char *module_name, EventNotifCb cb, const char *xpath, time_t start_time, time_t stop_time, sr_subscr_options_t opts)
 {
-    callback->private_data["oper_get_items"] = private_data;
-    cb_list.push_back(callback);
+    check_custom_loop_options(opts);
+    event_notif_cbs.push_back(cb);
 
     opts |= SR_SUBSCR_CTX_REUSE;
-    int ret = sr_oper_get_items_subscribe(_sess->_sess, module_name, path, oper_get_items_cb, callback->get(), opts, &_sub);
+    int ret = sr_event_notif_subscribe(
+            sess->_sess,
+            module_name,
+            xpath,
+            start_time,
+            stop_time,
+            [] (sr_session_ctx_t *session,
+                const sr_ev_notif_type_t notif_type,
+                const char *path,
+                const sr_val_t *values,
+                const size_t values_cnt,
+                time_t timestamp,
+                void *private_data)
+            {
+                auto sess = std::make_shared<Session>(session);
+                auto vals = std::make_shared<Vals>(values, values_cnt, nullptr);
+                auto cb = reinterpret_cast<EventNotifCb*>(private_data);
+                (*cb)(sess, notif_type, path, vals, timestamp);
+            },
+            (void*)&(event_notif_cbs.back()),
+            opts,
+            &ctx);
     if (ret != SR_ERR_OK) {
         throw_exception(ret);
     }
+    call_reg();
+}
+
+void Subscribe::event_notif_subscribe_tree(const char *module_name, EventNotifTreeCb cb, const char *xpath, time_t start_time, time_t stop_time, sr_subscr_options_t opts)
+{
+    check_custom_loop_options(opts);
+    event_notif_tree_cbs.push_back(cb);
+
+    opts |= SR_SUBSCR_CTX_REUSE;
+    int ret = sr_event_notif_subscribe_tree(
+            sess->_sess,
+            module_name,
+            xpath,
+            start_time,
+            stop_time,
+            [] (sr_session_ctx_t *session,
+                const sr_ev_notif_type_t notif_type,
+                const struct lyd_node *notif,
+                time_t timestamp,
+                void *private_data)
+            {
+                auto sess = std::make_shared<Session>(session);
+                auto node = std::make_shared<libyang::Data_Node>(const_cast<struct lyd_node *>(notif));
+                auto cb = reinterpret_cast<EventNotifTreeCb*>(private_data);
+                (*cb)(sess, notif_type, node, timestamp);
+            },
+            (void*)&(event_notif_tree_cbs.back()),
+            opts,
+            &ctx);
+    if (ret != SR_ERR_OK) {
+        throw_exception(ret);
+    }
+    call_reg();
+}
+
+void Subscribe::oper_get_items_subscribe(const char *module_name, OperGetItemsCb cb, const char *path, sr_subscr_options_t opts)
+{
+    check_custom_loop_options(opts);
+    oper_get_items_cbs.push_back(cb);
+
+    opts |= SR_SUBSCR_CTX_REUSE;
+    int ret = sr_oper_get_items_subscribe(
+            sess->_sess,
+            module_name,
+            path,
+            [] (sr_session_ctx_t *session,
+                const char *module_name,
+                const char *path,
+                const char *request_xpath,
+                uint32_t request_id,
+                struct lyd_node **parent,
+                void *private_data)
+            {
+                int ret;
+                libyang::S_Data_Node tree;
+                auto sess = std::make_shared<Session>(session);
+                auto cb = reinterpret_cast<OperGetItemsCb*>(private_data);
+                if (*parent) {
+                    tree = std::make_shared<libyang::Data_Node>(*parent);
+                    ret = (*cb)(sess, module_name, path, request_xpath, request_id, tree);
+                } else {
+                    ret = (*cb)(sess, module_name, path, request_xpath, request_id, tree);
+                    if (tree) {
+                        *parent = lyd_dup_withsiblings(tree->swig_node(), LYD_DUP_OPT_RECURSIVE);
+                    }
+                }
+
+                return ret;
+            },
+            (void*)&(oper_get_items_cbs.back()),
+            opts,
+            &ctx);
+    if (ret != SR_ERR_OK) {
+        throw_exception(ret);
+    }
+    call_reg();
 }
 
 int Subscribe::get_event_pipe()
 {
     int ret, ev_pipe;
 
-    ret = sr_get_event_pipe(_sub, &ev_pipe);
+    ret = sr_get_event_pipe(ctx, &ev_pipe);
     if (ret != SR_ERR_OK) {
         throw_exception(ret);
     }
@@ -603,22 +702,11 @@ time_t Subscribe::process_events(S_Session sess)
     int ret;
     time_t stop_time;
 
-    ret = sr_process_events(_sub, sess ? sess->_sess : nullptr, &stop_time);
+    ret = sr_process_events(ctx, sess ? sess->_sess : nullptr, &stop_time);
     if (ret != SR_ERR_OK) {
         throw_exception(ret);
     }
 
     return stop_time;
 }
-
-void Subscribe::unsubscribe()
-{
-    int ret = sr_unsubscribe(_sub);
-    if (ret != SR_ERR_OK) {
-        throw_exception(ret);
-    }
-
-    _sub = nullptr;
-}
-
 }
