@@ -23,78 +23,78 @@
 #define _DEFAULT_SOURCE /* mkstemps */
 #define _XOPEN_SOURCE 500 /* mkstemp */
 
+#include <errno.h>
+#include <getopt.h>
+#include <stdarg.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <unistd.h>
-#include <getopt.h>
-#include <stdarg.h>
 #include <unistd.h>
-#include <string.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <stdint.h>
-#include <errno.h>
 
 #include <libyang/libyang.h>
 
+#include "bin_common.h"
 #include "compat.h"
 #include "sysrepo.h"
-#include "bin_common.h"
 
 static void
 version_print(void)
 {
     printf(
-        "sysrepocfg - sysrepo configuration manipulation tool, compiled with libsysrepo v%s (SO v%s)\n"
-        "\n",
-        SR_VERSION, SR_SOVERSION
-    );
+            "sysrepocfg - sysrepo configuration manipulation tool, compiled with libsysrepo v%s (SO v%s)\n"
+            "\n",
+            SR_VERSION, SR_SOVERSION);
 }
 
 static void
 help_print(void)
 {
     printf(
-        "Usage:\n"
-        "  sysrepocfg <operation-option> [other-options]\n"
-        "\n"
-        "Available operation-options:\n"
-        "  -h, --help                   Prints usage help.\n"
-        "  -V, --version                Prints only information about sysrepo version.\n"
-        "  -I, --import[=<file-path>]   Import the configuration from a file or STDIN.\n"
-        "  -X, --export[=<file-path>]   Export configuration to a file or STDOUT.\n"
-        "  -E, --edit[=<file-path>/<editor>]\n"
-        "                               Edit configuration data by merging (applying) a configuration (edit) file or\n"
-        "                               by editing the current datastore content using a text editor.\n"
-        "  -R, --rpc[=<file-path>/<editor>]\n"
-        "                               Send a RPC/action in a file or using a text editor. Output is printed to STDOUT.\n"
-        "  -N, --notification[=<file-path>/<editor>]\n"
-        "                               Send a notification in a file or using a text editor.\n"
-        "  -C, --copy-from <file-path>/<source-datastore>\n"
-        "                               Perform a copy-config from a file or a datastore.\n"
-        "  -W, --new-data <file-path>   Set the configuration from a file as the initial one for a new module only scheduled\n"
-        "                               to be installed. Is useful for modules with mandatory top-level nodes.\n"
-        "\n"
-        "       When both a <file-path> and <editor>/<target-datastore> can be specified, it is always first checked\n"
-        "       that the file exists. If not, then it is interpreted as the other parameter.\n"
-        "       If no <file-path> and no <editor> is set, use text editor in $VISUAL or $EDITOR environment variables.\n"
-        "\n"
-        "Available other-options:\n"
-        "  -d, --datastore <datastore>  Datastore to be operated on, \"running\" by default (\"running\", \"startup\",\n"
-        "                               \"candidate\", or \"operational\") (import, export, edit, copy-from op).\n"
-        "  -m, --module <module-name>   Module to be operated on, otherwise it is operated on full datastore\n"
-        "                               (import, export, edit, copy-from, mandatory for new-data op).\n"
-        "  -x, --xpath <xpath>          XPath to select (export op).\n"
-        "  -f, --format <format>        Data format to be used, by default based on file extension or \"xml\" if not applicable\n"
-        "                               (\"xml\", \"json\", or \"lyb\") (import, export, edit, rpc, notification, copy-from, new-data op).\n"
-        "  -l, --lock                   Lock the specified datastore for the whole operation (edit op).\n"
-        "  -n, --not-strict             Silently ignore any unknown data (import, edit, rpc, notification, copy-from op).\n"
-        "  -p, --depth <number>         Limit the depth of returned subtrees, 0 so unlimited by default (export op).\n"
-        "  -t, --timeout <seconds>      Set the timeout for the operation, otherwise the default one is used.\n"
-        "  -w, --wait                   Wait for all the callbacks to be called on a data change including DONE or ABORT.\n"
-        "  -v, --verbosity <level>      Change verbosity to a level (none, error, warning, info, debug) or number (0, 1, 2, 3, 4).\n"
-        "\n"
-    );
+            "Usage:\n"
+            "  sysrepocfg <operation-option> [other-options]\n"
+            "\n"
+            "Available operation-options:\n"
+            "  -h, --help                   Prints usage help.\n"
+            "  -V, --version                Prints only information about sysrepo version.\n"
+            "  -I, --import[=<file-path>]   Import the configuration from a file or STDIN.\n"
+            "  -X, --export[=<file-path>]   Export configuration to a file or STDOUT.\n"
+            "  -E, --edit[=<file-path>/<editor>]\n"
+            "                               Edit configuration data by merging (applying) a configuration (edit) file or\n"
+            "                               by editing the current datastore content using a text editor.\n"
+            "  -R, --rpc[=<file-path>/<editor>]\n"
+            "                               Send a RPC/action in a file or using a text editor. Output is printed to STDOUT.\n"
+            "  -N, --notification[=<file-path>/<editor>]\n"
+            "                               Send a notification in a file or using a text editor.\n"
+            "  -C, --copy-from <file-path>/<source-datastore>\n"
+            "                               Perform a copy-config from a file or a datastore.\n"
+            "  -W, --new-data <file-path>   Set the configuration from a file as the initial one for a new module only scheduled\n"
+            "                               to be installed. Is useful for modules with mandatory top-level nodes.\n"
+            "\n"
+            "       When both a <file-path> and <editor>/<target-datastore> can be specified, it is always first checked\n"
+            "       that the file exists. If not, then it is interpreted as the other parameter.\n"
+            "       If no <file-path> and no <editor> is set, use text editor in $VISUAL or $EDITOR environment variables.\n"
+            "\n"
+            "Available other-options:\n"
+            "  -d, --datastore <datastore>  Datastore to be operated on, \"running\" by default (\"running\", \"startup\",\n"
+            "                               \"candidate\", or \"operational\") (import, export, edit, copy-from op).\n"
+            "  -m, --module <module-name>   Module to be operated on, otherwise it is operated on full datastore\n"
+            "                               (import, export, edit, copy-from, mandatory for new-data op).\n"
+            "  -x, --xpath <xpath>          XPath to select (export op).\n"
+            "  -f, --format <format>        Data format to be used, by default based on file extension or \"xml\" if not applicable\n"
+            "                               (\"xml\", \"json\", or \"lyb\") (import, export, edit, rpc, notification, copy-from, new-data op).\n"
+            "  -l, --lock                   Lock the specified datastore for the whole operation (edit op).\n"
+            "  -n, --not-strict             Silently ignore any unknown data (import, edit, rpc, notification, copy-from op).\n"
+            "  -p, --depth <number>         Limit the depth of returned subtrees, 0 so unlimited by default (export op).\n"
+            "  -t, --timeout <seconds>      Set the timeout for the operation, otherwise the default one is used.\n"
+            "  -w, --wait                   Wait for all the callbacks to be called on a data change including DONE or ABORT.\n"
+            "  -e, --defaults <wd-mode>     Print the default values, which are hidden by default (\"report-all\",\n"
+            "                               \"report-all-tagged\", \"trim\", \"explicit\", \"implicit-tagged\") (export, edit, rpc op).\n"
+            "  -v, --verbosity <level>      Change verbosity to a level (none, error, warning, info, debug) or number (0, 1, 2, 3, 4).\n"
+            "\n");
 }
 
 static void
@@ -187,6 +187,8 @@ step_read_file(FILE *file, char **mem)
         mem_used += fread(*mem + mem_used, 1, mem_size - mem_used, file);
     } while (mem_used == mem_size);
 
+    (*mem)[mem_used] = '\0';
+
     if (ferror(file)) {
         free(*mem);
         error_print(0, "Error reading from file (%s)", strerror(errno));
@@ -288,7 +290,7 @@ step_create_input_file(LYD_FORMAT format, char *tmp_file)
 
 static int
 op_import(sr_session_ctx_t *sess, const char *file_path, const char *module_name, LYD_FORMAT format, int not_strict,
-        int timeout_s, int wait)
+        int timeout_s)
 {
     struct lyd_node *data;
     int r, flags;
@@ -299,7 +301,7 @@ op_import(sr_session_ctx_t *sess, const char *file_path, const char *module_name
     }
 
     /* replace config (always spends data) */
-    r = sr_replace_config(sess, module_name, data, timeout_s * 1000, wait);
+    r = sr_replace_config(sess, module_name, data, timeout_s * 1000, 1);
     if (r) {
         error_print(r, "Replace config failed");
         return EXIT_FAILURE;
@@ -310,7 +312,7 @@ op_import(sr_session_ctx_t *sess, const char *file_path, const char *module_name
 
 static int
 op_export(sr_session_ctx_t *sess, const char *file_path, const char *module_name, const char *xpath, LYD_FORMAT format,
-        uint32_t max_depth, int timeout_s)
+        uint32_t max_depth, int wd_opt, int timeout_s)
 {
     struct lyd_node *data;
     FILE *file = NULL;
@@ -348,7 +350,7 @@ op_export(sr_session_ctx_t *sess, const char *file_path, const char *module_name
     }
 
     /* print exported data */
-    lyd_print_file(file ? file : stdout, data, format, LYP_FORMAT | LYP_WITHSIBLINGS);
+    lyd_print_file(file ? file : stdout, data, format, LYP_FORMAT | LYP_WITHSIBLINGS | wd_opt);
     lyd_free_withsiblings(data);
 
     /* cleanup */
@@ -360,7 +362,7 @@ op_export(sr_session_ctx_t *sess, const char *file_path, const char *module_name
 
 static int
 op_edit(sr_session_ctx_t *sess, const char *file_path, const char *editor, const char *module_name, LYD_FORMAT format,
-        int lock, int not_strict, int timeout_s, int wait)
+        int lock, int not_strict, int wd_opt, int timeout_s)
 {
     char tmp_file[22];
     int r, flags, rc = EXIT_FAILURE;
@@ -380,7 +382,7 @@ op_edit(sr_session_ctx_t *sess, const char *file_path, const char *editor, const
             return EXIT_FAILURE;
         }
 
-        r = sr_apply_changes(sess, timeout_s * 1000, wait);
+        r = sr_apply_changes(sess, timeout_s * 1000, 1);
         if (r != SR_ERR_OK) {
             error_print(r, "Failed to merge edit data");
             return EXIT_FAILURE;
@@ -401,7 +403,7 @@ op_edit(sr_session_ctx_t *sess, const char *file_path, const char *editor, const
     }
 
     /* use export operation to get data to edit */
-    if (op_export(sess, tmp_file, module_name, NULL, format, 0, timeout_s)) {
+    if (op_export(sess, tmp_file, module_name, NULL, format, 0, wd_opt, timeout_s)) {
         goto cleanup_unlock;
     }
 
@@ -411,7 +413,7 @@ op_edit(sr_session_ctx_t *sess, const char *file_path, const char *editor, const
     }
 
     /* use import operation to store edited data */
-    if (op_import(sess, tmp_file, module_name, format, not_strict, timeout_s, wait)) {
+    if (op_import(sess, tmp_file, module_name, format, not_strict, timeout_s)) {
         goto cleanup_unlock;
     }
 
@@ -426,7 +428,8 @@ cleanup_unlock:
 }
 
 static int
-op_rpc(sr_session_ctx_t *sess, const char *file_path, const char *editor, LYD_FORMAT format, int not_strict, int timeout_s)
+op_rpc(sr_session_ctx_t *sess, const char *file_path, const char *editor, LYD_FORMAT format, int not_strict, int wd_opt,
+        int timeout_s)
 {
     char tmp_file[22];
     int r, flags;
@@ -467,7 +470,7 @@ op_rpc(sr_session_ctx_t *sess, const char *file_path, const char *editor, LYD_FO
         }
     }
     if (node) {
-        lyd_print_file(stdout, output, format, LYP_FORMAT);
+        lyd_print_file(stdout, output, format, LYP_FORMAT | wd_opt);
     }
     lyd_free_withsiblings(output);
 
@@ -514,7 +517,7 @@ op_notif(sr_session_ctx_t *sess, const char *file_path, const char *editor, LYD_
 
 static int
 op_copy(sr_session_ctx_t *sess, const char *file_path, sr_datastore_t source_ds, const char *module_name,
-        LYD_FORMAT format, int not_strict, int timeout_s, int wait)
+        LYD_FORMAT format, int not_strict, int timeout_s)
 {
     int r, flags;
     struct lyd_node *data;
@@ -527,14 +530,14 @@ op_copy(sr_session_ctx_t *sess, const char *file_path, sr_datastore_t source_ds,
         }
 
         /* replace data */
-        r = sr_replace_config(sess, module_name, data, timeout_s * 1000, wait);
+        r = sr_replace_config(sess, module_name, data, timeout_s * 1000, 1);
         if (r) {
             error_print(r, "Replace config failed");
             return EXIT_FAILURE;
         }
     } else {
         /* copy config */
-        r = sr_copy_config(sess, module_name, source_ds, timeout_s * 1000, wait);
+        r = sr_copy_config(sess, module_name, source_ds, timeout_s * 1000, 1);
         if (r) {
             error_print(r, "Copy config failed");
             return EXIT_FAILURE;
@@ -585,7 +588,7 @@ arg_get_ds(const char *optarg, sr_datastore_t *ds)
 }
 
 int
-main(int argc, char** argv)
+main(int argc, char **argv)
 {
     sr_conn_ctx_t *conn = NULL;
     sr_session_ctx_t *sess = NULL;
@@ -594,7 +597,7 @@ main(int argc, char** argv)
     const char *module_name = NULL, *editor = NULL, *file_path = NULL, *xpath = NULL, *op_str;
     char *ptr;
     sr_log_level_t log_level = SR_LL_ERR;
-    int r, rc = EXIT_FAILURE, opt, operation = 0, lock = 0, not_strict = 0, timeout = 0, wait = 0;
+    int r, rc = EXIT_FAILURE, opt, operation = 0, lock = 0, not_strict = 0, timeout = 0, wd_opt = 0;
     uint32_t max_depth = 0;
     struct option options[] = {
         {"help",            no_argument,       NULL, 'h'},
@@ -614,7 +617,7 @@ main(int argc, char** argv)
         {"not-strict",      no_argument,       NULL, 'n'},
         {"depth",           required_argument, NULL, 'p'},
         {"timeout",         required_argument, NULL, 't'},
-        {"wait",            no_argument,       NULL, 'w'},
+        {"defaults",        required_argument, NULL, 'e'},
         {"verbosity",       required_argument, NULL, 'v'},
         {NULL,              0,                 NULL, 0},
     };
@@ -626,7 +629,7 @@ main(int argc, char** argv)
 
     /* process options */
     opterr = 0;
-    while ((opt = getopt_long(argc, argv, "hVI::X::E::R::N::C:W:d:m:x:f:lnp:t:wv:", options, NULL)) != -1) {
+    while ((opt = getopt_long(argc, argv, "hVI::X::E::R::N::C:W:d:m:x:f:lnp:t:e:v:", options, NULL)) != -1) {
         switch (opt) {
         case 'h':
             version_print();
@@ -778,8 +781,21 @@ main(int argc, char** argv)
                 goto cleanup;
             }
             break;
-        case 'w':
-            wait = 1;
+        case 'e':
+            if (!strcmp(optarg, "report-all")) {
+                wd_opt = LYP_WD_ALL;
+            } else if (!strcmp(optarg, "report-all-tagged")) {
+                wd_opt = LYP_WD_ALL_TAG;
+            } else if (!strcmp(optarg, "trim")) {
+                wd_opt = LYP_WD_TRIM;
+            } else if (!strcmp(optarg, "explicit")) {
+                wd_opt = LYP_WD_EXPLICIT;
+            } else if (!strcmp(optarg, "implicit-tagged")) {
+                wd_opt = LYP_WD_IMPL_TAG;
+            } else {
+                error_print(0, "Invalid defaults mode \"%s\"", optarg);
+                goto cleanup;
+            }
             break;
         case 'v':
             if (!strcmp(optarg, "none")) {
@@ -852,22 +868,22 @@ main(int argc, char** argv)
     /* perform the operation */
     switch (operation) {
     case 'I':
-        rc = op_import(sess, file_path, module_name, format, not_strict, timeout, wait);
+        rc = op_import(sess, file_path, module_name, format, not_strict, timeout);
         break;
     case 'X':
-        rc = op_export(sess, file_path, module_name, xpath, format, max_depth, timeout);
+        rc = op_export(sess, file_path, module_name, xpath, format, max_depth, wd_opt, timeout);
         break;
     case 'E':
-        rc = op_edit(sess, file_path, editor, module_name, format, lock, not_strict, timeout, wait);
+        rc = op_edit(sess, file_path, editor, module_name, format, lock, not_strict, wd_opt, timeout);
         break;
     case 'R':
-        rc = op_rpc(sess, file_path, editor, format, not_strict, timeout);
+        rc = op_rpc(sess, file_path, editor, format, not_strict, wd_opt, timeout);
         break;
     case 'N':
         rc = op_notif(sess, file_path, editor, format, not_strict);
         break;
     case 'C':
-        rc = op_copy(sess, file_path, source_ds, module_name, format, not_strict, timeout, wait);
+        rc = op_copy(sess, file_path, source_ds, module_name, format, not_strict, timeout);
         break;
     case 'W':
         if (!module_name) {
